@@ -113,7 +113,7 @@ from .parameter_value import (
     ListValueRef,
 )
 from .filters.alternative_filter import apply_alternative_filter_to_parameter_value_sq
-from .filters.scenario_filter import apply_scenario_filter_to_parameter_value_sq
+from .filters.scenario_filter import apply_scenario_filter_to_subqueries
 from .filters.tool_filter import apply_tool_filter_to_entity_sq
 from .filters.execution_filter import apply_execution_filter
 from .filters.renamer import apply_renaming_to_parameter_definition_sq, apply_renaming_to_entity_class_sq
@@ -128,4 +128,4 @@ from .filters.tools import (
 )
 
 name = "spinedb_api"
-__version__ = "0.24.0"
+__version__ = "0.28.0"
