@@ -17,7 +17,7 @@
 # TODO: improve docstrings
 
 from datetime import datetime
-from sqlalchemy import func, Table, Column, Integer, String, null
+from sqlalchemy import func, Table, Column, Integer, String, null, select
 from sqlalchemy.exc import DBAPIError
 from .exception import SpineDBAPIError
 from .helpers import get_relationship_entity_class_items, get_relationship_entity_items
@@ -29,9 +29,9 @@ class DatabaseMappingAddMixin:
     def __init__(self, *args, **kwargs):
         """Initialize class."""
         super().__init__(*args, **kwargs)
-        next_id = self._metadata.tables.get("next_id")
-        if next_id is None:
-            next_id = Table(
+        self._next_id = self._metadata.tables.get("next_id")
+        if self._next_id is None:
+            self._next_id = Table(
                 "next_id",
                 self._metadata,
                 Column("user", String(155), primary_key=True),
@@ -55,15 +55,27 @@ class DatabaseMappingAddMixin:
                 Column("entity_metadata_id", Integer, server_default=null()),
             )
             try:
-                next_id.create(self.connection)
+                self._next_id.create(self.connection)
             except DBAPIError:
                 # Some other concurrent process must have beaten us to create the table
-                next_id = Table("next_id", self._metadata, autoload=True)
-        self._next_id = next_id
+                self._next_id = Table("next_id", self._metadata, autoload=True)
 
     def _add_commit_id_and_ids(self, tablename, *items):
         if not items:
             return [], set()
+        ids = self._reserve_ids(tablename, len(items))
+        commit_id = self._make_commit_id()
+        for id_, item in zip(ids, items):
+            item["commit_id"] = commit_id
+            item["id"] = id_
+
+    def _reserve_ids(self, tablename, count):
+        if self.committing:
+            return self._do_reserve_ids(self.connection, tablename, count)
+        with self.engine.begin() as connection:
+            return self._do_reserve_ids(connection, tablename, count)
+
+    def _do_reserve_ids(self, connection, tablename, count):
         fieldname = {
             "object_class": "entity_class_id",
             "object": "entity_id",
@@ -85,8 +97,8 @@ class DatabaseMappingAddMixin:
             "parameter_value_metadata": "parameter_value_metadata_id",
             "entity_metadata": "entity_metadata_id",
         }[tablename]
-        commit_id = self.make_commit_id()  # This locks the SQLite DB file
-        next_id_row = self.query(self._next_id).one_or_none()
+        select_next_id = select([self._next_id])
+        next_id_row = connection.execute(select_next_id).first()
         if next_id_row is None:
             next_id = None
             stmt = self._next_id.insert()
@@ -96,17 +108,16 @@ class DatabaseMappingAddMixin:
         if next_id is None:
             table = self._metadata.tables[tablename]
             id_col = self.table_ids.get(tablename, "id")
-            max_id = self.query(func.max(getattr(table.c, id_col))).scalar()
+            select_max_id = select([func.max(getattr(table.c, id_col))])
+            max_id = connection.execute(select_max_id).scalar()
             next_id = max_id + 1 if max_id else 1
-        new_next_id = next_id + len(items)
-        self._checked_execute(stmt, {"user": self.username, "date": datetime.utcnow(), fieldname: new_next_id})
-        ids = list(range(next_id, new_next_id))
-        for id_, item in zip(ids, items):
-            item["commit_id"] = commit_id
-            item["id"] = id_
+        new_next_id = next_id + count
+        connection.execute(stmt, {"user": self.username, "date": datetime.utcnow(), fieldname: new_next_id})
+        return range(next_id, new_next_id)
 
     def _readd_items(self, tablename, *items):
         """Add known items to database."""
+        self._make_commit_id()
         for _ in self._do_add_items(tablename, *items):
             pass
 
@@ -187,15 +198,17 @@ class DatabaseMappingAddMixin:
         return self._metadata.tables[tablename]
 
     def _do_add_items(self, tablename, *items_to_add):
+        if not self.committing:
+            return
         items_to_add = tuple(self._items_with_type_id(tablename, *items_to_add))
         try:
             for tablename_, items_to_add_ in self._items_to_add_per_table(tablename, items_to_add):
                 table = self._get_table_for_insert(tablename_)
-                self._checked_execute(table.insert(), items_to_add_)
+                self._checked_execute(table.insert(), [{**item} for item in items_to_add_])
                 yield tablename_
         except DBAPIError as e:
             msg = f"DBAPIError while inserting {tablename} items: {e.orig.args}"
-            raise SpineDBAPIError(msg)
+            raise SpineDBAPIError(msg) from e
 
     def _items_to_add_per_table(self, tablename, items_to_add):
         """
@@ -252,20 +265,14 @@ class DatabaseMappingAddMixin:
         elif tablename == "parameter_definition":
             for item in items_to_add:
                 item["entity_class_id"] = (
-                    item.pop("object_class_id", None)
-                    or item.pop("relationship_class_id", None)
-                    or item.get("entity_class_id")
+                    item.get("object_class_id") or item.get("relationship_class_id") or item.get("entity_class_id")
                 )
             yield ("parameter_definition", items_to_add)
         elif tablename == "parameter_value":
             for item in items_to_add:
-                item["entity_id"] = (
-                    item.pop("object_id", None) or item.pop("relationship_id", None) or item.get("entity_id")
-                )
+                item["entity_id"] = item.get("object_id") or item.get("relationship_id") or item.get("entity_id")
                 item["entity_class_id"] = (
-                    item.pop("object_class_id", None)
-                    or item.pop("relationship_class_id", None)
-                    or item.get("entity_class_id")
+                    item.get("object_class_id") or item.get("relationship_class_id") or item.get("entity_class_id")
                 )
             yield ("parameter_value", items_to_add)
         else:
@@ -345,6 +352,8 @@ class DatabaseMappingAddMixin:
         added_metadata, errors = self.add_items(
             "metadata", *metadata_to_add, check=check, strict=strict, return_items=True, cache=cache
         )
+        for x in added_metadata:
+            cache.table_cache("metadata").add_item(x)
         if errors:
             return added_metadata, errors
         new_metadata_ids = {}
@@ -367,10 +376,8 @@ class DatabaseMappingAddMixin:
             if not return_items:
                 return added_metadata, metadata_errors
             return {i["id"] for i in added_metadata}, metadata_errors
-        # We want to invalidate cache just in case because it might be missing metadata
-        # records that may have been added above.
         added_item_metadata, item_errors = self.add_items(
-            table_name, *items, check=check, strict=strict, return_items=True, cache=None
+            table_name, *items, check=check, strict=strict, return_items=True, cache=cache
         )
         errors = metadata_errors + item_errors
         if not return_items:
