@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Database API.
 # Spine Database API is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
@@ -22,6 +22,7 @@ import logging
 import time
 from collections import Counter
 from types import MethodType
+from contextlib import contextmanager
 from sqlalchemy import create_engine, case, MetaData, Table, Column, false, and_, func, inspect, cast, Integer, or_
 from sqlalchemy.sql.expression import label, Alias
 from sqlalchemy.engine.url import make_url, URL
@@ -43,10 +44,10 @@ from .helpers import (
     group_concat,
     model_meta,
     copy_database_bind,
-    CacheItem,
 )
 from .filters.tools import pop_filter_configs
 from .spine_db_client import get_db_url_from_server
+from .db_cache import DBCache
 
 logging.getLogger("alembic").setLevel(logging.CRITICAL)
 
@@ -58,9 +59,39 @@ class DatabaseMappingBase:
     """
 
     _session_kwargs = {}
+    ITEM_TYPES = (
+        "object_class",
+        "relationship_class",
+        "parameter_value_list",
+        "list_value",
+        "parameter_definition",
+        "object",
+        "relationship",
+        "entity_group",
+        "parameter_value",
+        "alternative",
+        "scenario",
+        "scenario_alternative",
+        "feature",
+        "tool",
+        "tool_feature",
+        "tool_feature_method",
+        "metadata",
+        "entity_metadata",
+        "parameter_value_metadata",
+    )
 
     def __init__(
-        self, db_url, username=None, upgrade=False, codename=None, create=False, apply_filters=True, memory=False
+        self,
+        db_url,
+        username=None,
+        upgrade=False,
+        codename=None,
+        create=False,
+        apply_filters=True,
+        memory=False,
+        sqlite_timeout=1800,
+        advance_cache_query=None,
     ):
         """
         Args:
@@ -72,6 +103,8 @@ class DatabaseMappingBase:
             apply_filters (bool): Whether or not filters in the URL's query part are applied to the database map.
             memory (bool): Whether or not to use a sqlite memory db as replacement for this DB map.
         """
+        if advance_cache_query is None:
+            advance_cache_query = self._advance_cache_query
         # FIXME: We should also check the server memory property and use it here
         db_url = get_db_url_from_server(db_url)
         self.db_url = str(db_url)
@@ -86,8 +119,11 @@ class DatabaseMappingBase:
         self.username = username if username else "anon"
         self.codename = self._make_codename(codename)
         self._memory = memory
+        self.committing = True
         self._memory_dirty = False
-        self._original_engine = self.create_engine(self.sa_url, upgrade=upgrade, create=create)
+        self._original_engine = self.create_engine(
+            self.sa_url, upgrade=upgrade, create=create, sqlite_timeout=sqlite_timeout
+        )
         # NOTE: The NullPool is needed to receive the close event (or any events), for some reason
         self.engine = create_engine("sqlite://", poolclass=NullPool) if self._memory else self._original_engine
         self.connection = self.engine.connect()
@@ -98,6 +134,7 @@ class DatabaseMappingBase:
         self._metadata.reflect()
         self._tablenames = [t.name for t in self._metadata.sorted_tables]
         self.session = Session(self.connection, **self._session_kwargs)
+        self.cache = DBCache(advance_cache_query)
         # class and entity type id
         self._object_class_type = None
         self._relationship_class_type = None
@@ -128,6 +165,7 @@ class DatabaseMappingBase:
         self._metadata_sq = None
         self._parameter_value_metadata_sq = None
         self._entity_metadata_sq = None
+        self._clean_parameter_value_sq = None
         # Special convenience subqueries that join two or more tables
         self._ext_parameter_value_list_sq = None
         self._wide_parameter_value_list_sq = None
@@ -175,25 +213,25 @@ class DatabaseMappingBase:
         # Subqueries used to populate cache
         self.cache_sqs = {
             "entity": "entity_sq",
-            "feature": "ext_feature_sq",
+            "feature": "feature_sq",
             "tool": "tool_sq",
-            "tool_feature": "ext_tool_feature_sq",
-            "tool_feature_method": "ext_tool_feature_method_sq",
-            "parameter_value_list": "wide_parameter_value_list_sq",
+            "tool_feature": "tool_feature_sq",
+            "tool_feature_method": "tool_feature_method_sq",
+            "parameter_value_list": "parameter_value_list_sq",
             "list_value": "list_value_sq",
             "alternative": "alternative_sq",
-            "scenario": "wide_scenario_sq",
-            "scenario_alternative": "ext_linked_scenario_alternative_sq",
+            "scenario": "scenario_sq",
+            "scenario_alternative": "scenario_alternative_sq",
             "object_class": "object_class_sq",
-            "object": "ext_object_sq",
+            "object": "object_sq",
             "relationship_class": "wide_relationship_class_sq",
             "relationship": "wide_relationship_sq",
-            "entity_group": "ext_entity_group_sq",
-            "parameter_definition": "entity_parameter_definition_sq",
-            "parameter_value": "entity_parameter_value_sq",
+            "entity_group": "entity_group_sq",
+            "parameter_definition": "parameter_definition_sq",
+            "parameter_value": "clean_parameter_value_sq",
             "metadata": "metadata_sq",
-            "entity_metadata": "entity_metadata_sq",
-            "parameter_value_metadata": "parameter_value_metadata_sq",
+            "entity_metadata": "ext_entity_metadata_sq",
+            "parameter_value_metadata": "ext_parameter_value_metadata_sq",
             "commit": "commit_sq",
         }
         self.ancestor_tablenames = {
@@ -232,6 +270,39 @@ class DatabaseMappingBase:
         self.descendant_tablenames = {
             tablename: set(self._descendant_tablenames(tablename)) for tablename in self.cache_sqs
         }
+        self.object_class_type = None
+        self.relationship_class_type = None
+        self.object_entity_type = None
+        self.relationship_entity_type = None
+
+    def _init_type_attributes(self):
+        self.object_class_type = (
+            self.query(self.entity_class_type_sq).filter(self.entity_class_type_sq.c.name == "object").first().id
+        )
+        self.relationship_class_type = (
+            self.query(self.entity_class_type_sq).filter(self.entity_class_type_sq.c.name == "relationship").first().id
+        )
+        self.object_entity_type = (
+            self.query(self.entity_type_sq).filter(self.entity_type_sq.c.name == "object").first().id
+        )
+        self.relationship_entity_type = (
+            self.query(self.entity_type_sq).filter(self.entity_type_sq.c.name == "relationship").first().id
+        )
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, _exc_type, _exc_val, _exc_tb):
+        self.connection.close()
+
+    @contextmanager
+    def override_committing(self, new_committing):
+        committing = self.committing
+        self.committing = new_committing
+        try:
+            yield None
+        finally:
+            self.committing = committing
 
     def _descendant_tablenames(self, tablename):
         child_tablenames = {
@@ -256,7 +327,22 @@ class DatabaseMappingBase:
                     yield child
                     yield from self._descendant_tablenames(child)
 
-    def make_commit_id(self):
+    def sorted_tablenames(self):
+        tablenames = list(self.ITEM_TYPES)
+        sorted_tablenames = []
+        while tablenames:
+            tablename = tablenames.pop(0)
+            ancestors = self.ancestor_tablenames.get(tablename)
+            if ancestors is None or all(x in sorted_tablenames for x in ancestors):
+                sorted_tablenames.append(tablename)
+            else:
+                tablenames.append(tablename)
+        return sorted_tablenames
+
+    def commit_id(self):
+        return self._commit_id
+
+    def _make_commit_id(self):
         return None
 
     def _check_commit(self, comment):
@@ -282,7 +368,7 @@ class DatabaseMappingBase:
         return hashing.hexdigest()
 
     @staticmethod
-    def create_engine(sa_url, upgrade=False, create=False):
+    def create_engine(sa_url, upgrade=False, create=False, sqlite_timeout=1800):
         """Create engine.
 
         Args
@@ -294,7 +380,7 @@ class DatabaseMappingBase:
             Engine
         """
         if sa_url.drivername == "sqlite":
-            connect_args = {'timeout': 1800}
+            connect_args = {'timeout': sqlite_timeout}
         else:
             connect_args = {}
         try:
@@ -411,9 +497,13 @@ class DatabaseMappingBase:
         """Set to `None` subquery attributes involving the affected tables.
         This forces the subqueries to be refreshed when the corresponding property is accessed.
         """
-        attrs = set(attr for table in tablenames for attr in self._get_table_to_sq_attr().get(table, []))
-        for attr in attrs:
-            setattr(self, attr, None)
+        tablenames = list(tablenames)
+        for tablename in tablenames:
+            if self.cache.pop(tablename, None):
+                self._do_advance_cache_query(tablename)
+        attr_names = set(attr for tablename in tablenames for attr in self._get_table_to_sq_attr().get(tablename, []))
+        for attr_name in attr_names:
+            setattr(self, attr_name, None)
 
     def query(self, *args, **kwargs):
         """Return a sqlalchemy :class:`~sqlalchemy.orm.query.Query` object applied
@@ -461,51 +551,21 @@ class DatabaseMappingBase:
     @property
     def alternative_sq(self):
         if self._alternative_sq is None:
-            self._alternative_sq = self._subquery("alternative")
+            self._alternative_sq = self._make_alternative_sq()
         return self._alternative_sq
 
     @property
     def scenario_sq(self):
         if self._scenario_sq is None:
-            self._scenario_sq = self._subquery("scenario")
+            self._scenario_sq = self._make_scenario_sq()
         return self._scenario_sq
 
     @property
     def scenario_alternative_sq(self):
         if self._scenario_alternative_sq is None:
-            self._scenario_alternative_sq = self._subquery("scenario_alternative")
+            self._scenario_alternative_sq = self._make_scenario_alternative_sq()
         return self._scenario_alternative_sq
 
-    @property
-    def object_class_type(self):
-        if self._object_class_type is None:
-            result = self.query(self.entity_class_type_sq).filter(self.entity_class_type_sq.c.name == "object").first()
-            self._object_class_type = result.id
-        return self._object_class_type
-
-    @property
-    def relationship_class_type(self):
-        if self._relationship_class_type is None:
-            result = (
-                self.query(self.entity_class_type_sq).filter(self.entity_class_type_sq.c.name == "relationship").first()
-            )
-            self._relationship_class_type = result.id
-        return self._relationship_class_type
-
-    @property
-    def object_entity_type(self):
-        if self._object_entity_type is None:
-            result = self.query(self.entity_type_sq).filter(self.entity_type_sq.c.name == "object").first()
-            self._object_entity_type = result.id
-        return self._object_entity_type
-
-    @property
-    def relationship_entity_type(self):
-        if self._relationship_entity_type is None:
-            result = self.query(self.entity_type_sq).filter(self.entity_type_sq.c.name == "relationship").first()
-            self._relationship_entity_type = result.id
-        return self._relationship_entity_type
-
     @property
     def entity_class_type_sq(self):
         """A subquery of the form:
@@ -688,7 +748,20 @@ class DatabaseMappingBase:
             sqlalchemy.sql.expression.Alias
         """
         if self._entity_group_sq is None:
-            self._entity_group_sq = self._subquery("entity_group")
+            group_entity = aliased(self.entity_sq)
+            member_entity = aliased(self.entity_sq)
+            entity_group_sq = self._subquery("entity_group")
+            self._entity_group_sq = (
+                self.query(
+                    entity_group_sq.c.id,
+                    entity_group_sq.c.entity_class_id,
+                    group_entity.c.id.label("entity_id"),
+                    member_entity.c.id.label("member_id"),
+                )
+                .join(group_entity, group_entity.c.id == entity_group_sq.c.entity_id)
+                .join(member_entity, member_entity.c.id == entity_group_sq.c.member_id)
+                .subquery()
+            )
         return self._entity_group_sq
 
     @property
@@ -722,6 +795,21 @@ class DatabaseMappingBase:
             self._parameter_value_sq = self._make_parameter_value_sq()
         return self._parameter_value_sq
 
+    @property
+    def clean_parameter_value_sq(self):
+        """A subquery of the parameter_value table that excludes rows with filtered entities.
+        This yields the correct results whenever there are both a scenario filter that filters some parameter values,
+        and a tool filter that then filters some entities based on the value of some their parameters
+        after the scenario filtering. Mildly insane.
+        """
+        if self._clean_parameter_value_sq is None:
+            self._clean_parameter_value_sq = (
+                self.query(self.parameter_value_sq)
+                .join(self.entity_sq, self.entity_sq.c.id == self.parameter_value_sq.c.entity_id)
+                .subquery()
+            )
+        return self._clean_parameter_value_sq
+
     @property
     def parameter_value_list_sq(self):
         """A subquery of the form:
@@ -1617,6 +1705,7 @@ class DatabaseMappingBase:
                     self.alternative_sq.c.name.label("alternative_name"),
                     self.metadata_sq.c.name.label("metadata_name"),
                     self.metadata_sq.c.value.label("metadata_value"),
+                    self.parameter_value_metadata_sq.c.commit_id,
                 )
                 .filter(self.parameter_value_metadata_sq.c.parameter_value_id == self.parameter_value_sq.c.id)
                 .filter(self.parameter_value_sq.c.parameter_definition_id == self.parameter_definition_sq.c.id)
@@ -1642,6 +1731,7 @@ class DatabaseMappingBase:
                     self.entity_sq.c.name.label("entity_name"),
                     self.metadata_sq.c.name.label("metadata_name"),
                     self.metadata_sq.c.value.label("metadata_value"),
+                    self.entity_metadata_sq.c.commit_id,
                 )
                 .filter(self.entity_metadata_sq.c.entity_id == self.entity_sq.c.id)
                 .filter(self.entity_metadata_sq.c.metadata_id == self.metadata_sq.c.id)
@@ -1738,6 +1828,33 @@ class DatabaseMappingBase:
             .subquery()
         )
 
+    def _make_alternative_sq(self):
+        """
+        Creates a subquery for alternatives.
+
+        Returns:
+            Alias: an alternative subquery
+        """
+        return self._subquery("alternative")
+
+    def _make_scenario_sq(self):
+        """
+        Creates a subquery for scenarios.
+
+        Returns:
+            Alias: a scenario subquery
+        """
+        return self._subquery("scenario")
+
+    def _make_scenario_alternative_sq(self):
+        """
+        Creates a subquery for scenario alternatives.
+
+        Returns:
+            Alias: a scenario alternative subquery
+        """
+        return self._subquery("scenario_alternative")
+
     def get_import_alternative(self, cache=None):
         """Returns the id of the alternative to use as default for all import operations.
 
@@ -1750,7 +1867,7 @@ class DatabaseMappingBase:
 
     def _create_import_alternative(self, cache=None):
         """Creates the alternative to be used as default for all import operations."""
-        if cache is None:
+        if "alternative" not in cache:
             cache = self.make_cache({"alternative"})
         self._import_alternative_name = "Base"
         self._import_alternative_id = next(
@@ -1835,6 +1952,54 @@ class DatabaseMappingBase:
         self._create_import_alternative = MethodType(method, self)
         self._import_alternative_id = None
 
+    def override_alternative_sq_maker(self, method):
+        """
+        Overrides the function that creates the ``alternative_sq`` property.
+
+        Args:
+            method (Callable): a function that accepts a :class:`DatabaseMappingBase` as its argument and
+                returns alternative subquery as an :class:`Alias` object
+        """
+        self._make_alternative_sq = MethodType(method, self)
+        self._clear_subqueries("alternative")
+
+    def restore_alternative_sq_maker(self):
+        """Restores the original function that creates the ``alternative_sq`` property."""
+        self._make_alternative_sq = MethodType(DatabaseMappingBase._make_alternative_sq, self)
+        self._clear_subqueries("alternative")
+
+    def override_scenario_sq_maker(self, method):
+        """
+        Overrides the function that creates the ``scenario_sq`` property.
+
+        Args:
+            method (Callable): a function that accepts a :class:`DatabaseMappingBase` as its argument and
+                returns scenario subquery as an :class:`Alias` object
+        """
+        self._make_scenario_sq = MethodType(method, self)
+        self._clear_subqueries("scenario")
+
+    def restore_scenario_sq_maker(self):
+        """Restores the original function that creates the ``scenario_sq`` property."""
+        self._make_scenario_sq = MethodType(DatabaseMappingBase._make_scenario_sq, self)
+        self._clear_subqueries("scenario")
+
+    def override_scenario_alternative_sq_maker(self, method):
+        """
+        Overrides the function that creates the ``scenario_alternative_sq`` property.
+
+        Args:
+            method (Callable): a function that accepts a :class:`DatabaseMappingBase` as its argument and
+                returns scenario alternative subquery as an :class:`Alias` object
+        """
+        self._make_scenario_alternative_sq = MethodType(method, self)
+        self._clear_subqueries("scenario_alternative")
+
+    def restore_scenario_alternative_sq_maker(self):
+        """Restores the original function that creates the ``scenario_alternative_sq`` property."""
+        self._make_scenario_alternative_sq = MethodType(DatabaseMappingBase._make_scenario_alternative_sq, self)
+        self._clear_subqueries("scenario_alternative")
+
     def _checked_execute(self, stmt, items):
         if not items:
             return
@@ -1856,268 +2021,34 @@ class DatabaseMappingBase:
             self.connection.execute(table.delete())
         self.connection.execute("INSERT INTO alternative VALUES (1, 'Base', 'Base alternative', null)")
 
-    def make_cache(self, tablenames, only_descendants=False, include_ancestors=False, forced_table_names=None):
-        if only_descendants:
-            tablenames = {
+    def make_cache(self, tablenames, include_descendants=False, include_ancestors=False, force_tablenames=None):
+        if include_descendants:
+            tablenames |= {
                 descendant for tablename in tablenames for descendant in self.descendant_tablenames.get(tablename, ())
             }
         if include_ancestors:
             tablenames |= {
                 ancestor for tablename in tablenames for ancestor in self.ancestor_tablenames.get(tablename, ())
             }
-        if forced_table_names:
-            tablenames |= forced_table_names
-        return {
-            tablename: {x.id: CacheItem(**x._asdict()) for x in self.query(getattr(self, self.cache_sqs[tablename]))}
-            for tablename in tablenames & self.cache_sqs.keys()
-        }
+        if force_tablenames:
+            tablenames |= force_tablenames
+        for tablename in tablenames & self.cache_sqs.keys():
+            self._do_advance_cache_query(tablename)
+        return self.cache
 
-    @staticmethod
-    def cache_to_db(tablename, item):
-        """
-        Returns the db equivalent of a cache item.
+    def _advance_cache_query(self, tablename, callback=None):
+        advanced = False
+        if tablename not in self.cache:
+            advanced = True
+            self._do_advance_cache_query(tablename)
+        if callback is not None:
+            callback()
+        return advanced
 
-        Args:
-            tablename (str): The table name
-            item (dict): The item in the cache
-
-        Returns:
-            dict
-        """
-        if tablename == "relationship_class":
-            return DatabaseMappingBase.cache_relationship_class_to_db(item)
-        if tablename == "relationship":
-            return DatabaseMappingBase.cache_relationship_to_db(item)
-        if tablename == "parameter_definition":
-            return DatabaseMappingBase.cache_parameter_definition_to_db(item)
-        if tablename == "parameter_value":
-            return DatabaseMappingBase.cache_parameter_value_to_db(item)
-        if tablename == "list_value":
-            return DatabaseMappingBase.cache_list_value_to_db(item)
-        if tablename == "entity_group":
-            return DatabaseMappingBase.cache_entity_group_to_db(item)
-        return item.copy()
-
-    @staticmethod
-    def cache_relationship_class_to_db(item):
-        return {
-            "id": item["id"],
-            "name": item["name"],
-            "description": item.get("description"),
-            "display_icon": item.get("display_icon"),
-            "object_class_id_list": tuple(int(id_) for id_ in item["object_class_id_list"].split(",")),
-            "commit_id": item["commit_id"],
-        }
-
-    @staticmethod
-    def cache_relationship_to_db(item):
-        return {
-            "id": item["id"],
-            "name": item["name"],
-            "class_id": item["class_id"],
-            "object_class_id_list": tuple(int(id_) for id_ in item["object_class_id_list"].split(",")),
-            "object_id_list": tuple(int(id_) for id_ in item["object_id_list"].split(",")),
-            "commit_id": item["commit_id"],
-        }
-
-    @staticmethod
-    def cache_parameter_definition_to_db(item):
-        return {
-            "id": item["id"],
-            "entity_class_id": item["entity_class_id"],
-            "name": item["parameter_name"],
-            "parameter_value_list_id": item.get("value_list_id"),
-            "default_value": item.get("default_value"),
-            "default_type": item.get("default_type"),
-            "description": item.get("description"),
-            "commit_id": item["commit_id"],
-        }
-
-    @staticmethod
-    def cache_parameter_value_to_db(item):
-        return {
-            "id": item["id"],
-            "entity_class_id": item["entity_class_id"],
-            "entity_id": item["entity_id"],
-            "parameter_definition_id": item["parameter_id"],
-            "alternative_id": item["alternative_id"],
-            "value": item["value"],
-            "type": item["type"],
-            "commit_id": item["commit_id"],
-        }
-
-    @staticmethod
-    def cache_list_value_to_db(item):
-        return {
-            "id": item["id"],
-            "parameter_value_list_id": item["parameter_value_list_id"],
-            "index": item["index"],
-            "type": item["type"],
-            "value": item["value"],
-            "commit_id": item["commit_id"],
-        }
-
-    @staticmethod
-    def cache_entity_group_to_db(item):
-        return {
-            "id": item["id"],
-            "entity_class_id": item["class_id"],
-            "entity_id": item["group_id"],
-            "member_id": item["member_id"],
-        }
-
-    def _get_item(self, cache, tablename, id_):
-        table_cache = cache.get(tablename, {})
-        item = table_cache.get(id_, {})
-        if item:
-            return item
-        table_cache.update(self.make_cache({tablename})[tablename])
-        return table_cache.get(id_, {})
-
-    def _get_item_by_field(self, cache, tablename, field, value):
-        table_cache = cache.get(tablename, {})
-        item = next(iter(x for x in table_cache.values() if x.get(field) == value), {})
-        if item:
-            return item
-        table_cache.update(self.make_cache({tablename})[tablename])
-        return next(iter(x for x in table_cache.values() if x.get(field) == value), {})
-
-    def db_to_cache(self, cache, tablename, item):
-        """
-        Returns the cache equivalent of a db item.
-
-        Args:
-            db_map (DiffDatabaseMapping): the db map
-            tablename (str): The item type
-            item (dict): The item in the db
-
-        Returns:
-            dict
-        """
-
-        item = item.copy()
-        if tablename == "object_class":
-            item["display_icon"] = item.get("display_icon")
-        elif tablename == "object":
-            item["class_name"] = self._get_item(cache, "object_class", item["class_id"])["name"]
-            item["group_id"] = self._get_item_by_field(cache, "entity_group", "entity_id", item["id"]).get("entity_id")
-        elif tablename == "relationship_class":
-            item["object_class_name_list"] = ",".join(
-                self._get_item(cache, "object_class", id_)["name"] for id_ in item["object_class_id_list"]
-            )
-            item["object_class_id_list"] = ",".join(str(id_) for id_ in item["object_class_id_list"])
-            item["display_icon"] = item.get("display_icon")
-        elif tablename == "relationship":
-            item["class_name"] = self._get_item(cache, "relationship_class", item["class_id"])["name"]
-            item["object_name_list"] = ",".join(
-                self._get_item(cache, "object", id_)["name"] for id_ in item["object_id_list"]
-            )
-            item["object_id_list"] = ",".join(str(id_) for id_ in item["object_id_list"])
-            item["object_class_name_list"] = ",".join(
-                self._get_item(cache, "object_class", id_)["name"] for id_ in item["object_class_id_list"]
-            )
-            item["object_class_id_list"] = ",".join(str(id_) for id_ in item["object_class_id_list"])
-        elif tablename == "parameter_definition":
-            item["parameter_name"] = item.pop("name", item.get("parameter_name"))
-            object_class = self._get_item(cache, "object_class", item["entity_class_id"])
-            relationship_class = self._get_item(cache, "relationship_class", item["entity_class_id"])
-            item["entity_class_name"] = object_class.get("name") or relationship_class.get("name")
-            item["object_class_id"] = object_class.get("id")
-            item["object_class_name"] = object_class.get("name")
-            item["relationship_class_id"] = relationship_class.get("id")
-            item["relationship_class_name"] = relationship_class.get("name")
-            item["object_class_id_list"] = relationship_class.get("object_class_id_list")
-            item["object_class_name_list"] = relationship_class.get("object_class_name_list")
-            item["value_list_id"] = value_list_id = item.pop("parameter_value_list_id", item.get("value_list_id"))
-            item["value_list_name"] = self._get_item(cache, "parameter_value_list", value_list_id).get("name")
-            if item.get("default_type") == "list_value_ref":
-                item["list_value_id"] = list_value_id = int(item["default_value"])
-                list_value_item = self._get_item(cache, "list_value", list_value_id)
-                item["default_value"] = list_value_item["value"]
-                item["default_type"] = list_value_item["type"]
-            else:
-                item["default_value"] = item.get("default_value")
-                item["default_type"] = item.get("default_type")
-                item["list_value_id"] = None
-            item["description"] = item.get("description")
-            item.pop("parsed_value", None)
-        elif tablename == "parameter_value":
-            item["parameter_id"] = parameter_id = item.pop("parameter_definition_id", item.get("parameter_id"))
-            param_def = self._get_item(cache, "parameter_definition", parameter_id)
-            item["parameter_name"] = param_def["parameter_name"]
-            item["entity_class_id"] = param_def["entity_class_id"]
-            item["object_class_id"] = object_class_id = param_def["object_class_id"]
-            item["relationship_class_id"] = relationship_class_id = param_def["relationship_class_id"]
-            item["object_class_name"] = param_def["object_class_name"]
-            item["relationship_class_name"] = param_def["relationship_class_name"]
-            item["object_class_id_list"] = param_def["object_class_id_list"]
-            item["object_class_name_list"] = param_def["object_class_name_list"]
-            item["object_id"] = object_id = item["entity_id"] if object_class_id else None
-            object_ = self._get_item(cache, "object", object_id)
-            item["object_name"] = object_.get("name")
-            item["relationship_id"] = relationship_id = item["entity_id"] if relationship_class_id else None
-            relationship = self._get_item(cache, "relationship", relationship_id)
-            item["object_id_list"] = relationship.get("object_id_list")
-            item["object_name_list"] = relationship.get("object_name_list")
-            item["alternative_name"] = self._get_item(cache, "alternative", item["alternative_id"])["name"]
-            if item["type"] == "list_value_ref":
-                item["list_value_id"] = list_value_id = int(item["value"])
-                list_value_item = self._get_item(cache, "list_value", list_value_id)
-                item["value"] = list_value_item["value"]
-                item["type"] = list_value_item["type"]
-            else:
-                item["list_value_id"] = None
-            item.pop("parsed_value", None)
-        elif tablename == "entity_group":
-            item["class_id"] = item["entity_class_id"]
-            item["group_id"] = item["entity_id"]
-            item["class_name"] = (
-                self._get_item(cache, "object_class", item["class_id"])
-                or self._get_item(cache, "relationship_class", item["class_id"])["name"]
-            )
-            item["group_name"] = (
-                self._get_item(cache, "object", item["group_id"])
-                or self._get_item(cache, "relationship", item["group_id"])
-            )["name"]
-            item["member_name"] = (
-                self._get_item(cache, "object", item["member_id"])
-                or self._get_item(cache, "relationship", item["member_id"])
-            )["name"]
-        elif tablename == "scenario":
-            item["active"] = item.get("active", False)
-        elif tablename == "feature":
-            param_def = self._get_item(cache, "parameter_definition", item["parameter_definition_id"])
-            item["parameter_definition_name"] = param_def["parameter_name"]
-            item["entity_class_id"] = entity_class_id = self._get_item(cache, "parameter_definition", param_def["id"])[
-                "entity_class_id"
-            ]
-            item["entity_class_name"] = self._get_item(cache, "object_class", entity_class_id).get(
-                "name"
-            ) or self._get_item(cache, "relationship_class", entity_class_id).get("name")
-            item["parameter_value_list_name"] = self._get_item(
-                cache, "parameter_value_list", item["parameter_value_list_id"]
-            ).get("name")
-        elif tablename == "tool_feature":
-            feature = self._get_item(cache, "feature", item["feature_id"])
-            tool = self._get_item(cache, "tool", item["tool_id"])
-            par_val_lst = self._get_item(cache, "parameter_value_list", item["parameter_value_list_id"])
-            item["entity_class_id"] = feature["entity_class_id"]
-            item["entity_class_name"] = feature["entity_class_name"]
-            item["parameter_definition_id"] = feature["parameter_definition_id"]
-            item["parameter_definition_name"] = feature["parameter_definition_name"]
-            item["tool_name"] = tool["name"]
-            item["parameter_value_list_name"] = par_val_lst["name"]
-            item["required"] = item.get("required", False)
-        elif tablename == "list_value":
-            item.pop("parsed_value", None)
-        return item
-
-    def db_to_db(self, cache, tablename, item):
-        if tablename == "relationship":
-            item["object_class_id_list"] = [
-                self._get_item(cache, "object", id_).get("class_id") for id_ in item["object_id_list"]
-            ]
-        return item
+    def _do_advance_cache_query(self, tablename):
+        table_cache = self.cache.table_cache(tablename)
+        for x in self.query(getattr(self, self.cache_sqs[tablename])).yield_per(1000).enable_eagerloads(False):
+            table_cache.add_item(x._asdict())
 
     def _items_with_type_id(self, tablename, *items):
         type_id = {
@@ -2231,3 +2162,6 @@ class DatabaseMappingBase:
         table.drop(self.connection, checkfirst=True)
         table.create(self.connection)
         return table
+
+    def get_filter_configs(self):
+        return self._filter_configs
