@@ -943,21 +943,6 @@ def fix_name_ambiguity(input_list, offset=0, prefix=""):
     return result
 
 
-class CacheItem(dict):
-    """A dictionary that behaves kinda like a row from a query result.
-
-    It is used to store items in a cache, so we can access them as if they were rows from a query result.
-    This is mainly because we want to use the cache as a replacement for db queries in some methods.
-    """
-
-    def __getattr__(self, name):
-        """Overridden method to return the dictionary key named after the attribute, or None if it doesn't exist."""
-        return self.get(name)
-
-    def _asdict(self):
-        return dict(**self)
-
-
 def vacuum(url):
     engine = create_engine(url)
     if not engine.url.drivername.startswith("sqlite"):
