@@ -133,7 +133,9 @@ class _DBServerManager:
         server_thread = threading.Thread(target=server.serve_forever)
         server_thread.daemon = True
         server_thread.start()
-        SpineDBClient(server.server_address).open_db_map(db_url, upgrade, memory)
+        error = SpineDBClient(server.server_address).open_db_map(db_url, upgrade, memory).get("error")
+        if error:
+            raise RuntimeError(error)
         return server.server_address
 
     def _shutdown_server(self, server_address):
@@ -199,9 +201,10 @@ class _ManagerRequestHandler:
         self._mngr_queue = mngr_queue
 
     def _run_request(self, request, *args, **kwargs):
-        output_queue = mp.Manager().Queue()
-        self._mngr_queue.put((output_queue, request, args, kwargs))
-        return output_queue.get()
+        with mp.Manager() as manager:
+            output_queue = manager.Queue()
+            self._mngr_queue.put((output_queue, request, args, kwargs))
+            return output_queue.get()
 
     def start_server(self, db_url, upgrade, memory, ordering):
         return self._run_request("start_server", db_url, upgrade, memory, ordering)
@@ -273,6 +276,7 @@ class _DBWorker:
             request, args, kwargs = input_
             handler = {
                 "query": self._do_query,
+                "filtered_query": self._do_filtered_query,
                 "import_data": self._do_import_data,
                 "export_data": self._do_export_data,
                 "call_method": self._do_call_method,
@@ -296,6 +300,18 @@ class _DBWorker:
             result[sq_name] = [x._asdict() for x in self._db_map.query(sq)]
         return dict(result=result)
 
+    def _do_filtered_query(self, **kwargs):
+        result = {}
+        for sq_name, filters in kwargs.items():
+            sq = getattr(self._db_map, sq_name, None)
+            if sq is None:
+                continue
+            qry = self._db_map.query(sq)
+            for field, value in filters.items():
+                qry = qry.filter_by(**{field: value})
+            result[sq_name] = [x._asdict() for x in qry]
+        return dict(result=result)
+
     def _do_import_data(self, data, comment):
         count, errors = import_data(self._db_map, unparse_value=_unparse_value, **data)
         if count and comment:
@@ -318,11 +334,17 @@ class _DBWorker:
         self._db_map.restore_entity_class_sq_maker()
         self._db_map.restore_parameter_definition_sq_maker()
         self._db_map.restore_parameter_value_sq_maker()
+        self._db_map.restore_alternative_sq_maker()
+        self._db_map.restore_scenario_sq_maker()
+        self._db_map.restore_scenario_alternative_sq_maker()
         return dict(result=True)
 
     def _do_apply_filters(self, configs):
-        apply_filter_stack(self._db_map, configs)
-        return dict(result=True)
+        try:
+            apply_filter_stack(self._db_map, configs)
+            return dict(result=True)
+        except Exception as error:  # pylint: disable=broad-except
+            return dict(error=str(error))
 
 
 class _DBManager:
@@ -360,6 +382,9 @@ class _DBManager:
     def query(self, server_address, *args):
         return self._run_request(server_address, "query", args=args)
 
+    def filtered_query(self, server_address, **kwargs):
+        return self._run_request(server_address, "filtered_query", kwargs=kwargs)
+
     def import_data(self, server_address, data, comment):
         return self._run_request(server_address, "import_data", args=(data, comment))
 
@@ -396,6 +421,15 @@ class HandleDBMixin:
         """
         return _db_manager.query(self.server_address, *args)
 
+    def filtered_query(self, **kwargs):
+        """
+        Runs queries with filters.
+
+        Returns:
+            dict: where result is a dict from subquery name to a list of items from thay subquery, if successful.
+        """
+        return _db_manager.filtered_query(self.server_address, **kwargs)
+
     def import_data(self, data, comment):
         """Imports data and commit.
 
@@ -474,6 +508,7 @@ class HandleDBMixin:
             return dict(error=1, result=_required_client_version)
         handler = {
             "query": self.query,
+            "filtered_query": self.filtered_query,
             "import_data": self.import_data,
             "export_data": self.export_data,
             "call_method": self.call_method,
@@ -500,7 +535,9 @@ class HandleDBMixin:
 class DBHandler(HandleDBMixin):
     def __init__(self, db_url, upgrade=False, memory=False):
         self.server_address = uuid.uuid4().hex
-        _db_manager.open_db_map(self.server_address, db_url, upgrade, memory)
+        error = _db_manager.open_db_map(self.server_address, db_url, upgrade, memory).get("error")
+        if error:
+            raise RuntimeError(error)
         atexit.register(self.close)
 
     def close(self):
