@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Database API.
 # Spine Database API is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
@@ -24,6 +24,7 @@ from sqlalchemy.engine.url import make_url, URL
 from sqlalchemy.util import KeyedTuple
 from spinedb_api.diff_db_mapping import DiffDatabaseMapping
 from spinedb_api.exception import SpineIntegrityError
+from spinedb_api.db_cache import DBCache
 from spinedb_api import import_functions, SpineDBAPIError
 
 
@@ -997,6 +998,7 @@ class TestDiffDatabaseMappingAdd(unittest.TestCase):
                 "metadata_value": "My metadata.",
                 "metadata_id": 1,
                 "id": 1,
+                "commit_id": 3,
             },
         )
 
@@ -1022,11 +1024,14 @@ class TestDiffDatabaseMappingAdd(unittest.TestCase):
                 "metadata_value": "My metadata.",
                 "metadata_id": 1,
                 "id": 1,
+                "commit_id": 3,
             },
         )
 
     def test_add_entity_metadata_doesnt_raise_with_empty_cache(self):
-        items, errors = self._db_map.add_entity_metadata({"entity_id": 1, "metadata_id": 1}, cache={}, strict=False)
+        items, errors = self._db_map.add_entity_metadata(
+            {"entity_id": 1, "metadata_id": 1}, cache=DBCache(lambda *args, **kwargs: None), strict=False
+        )
         self.assertEqual(items, set())
         self.assertEqual(len(errors), 1)
 
@@ -1051,6 +1056,7 @@ class TestDiffDatabaseMappingAdd(unittest.TestCase):
                 "metadata_value": "object metadata",
                 "metadata_id": 1,
                 "id": 1,
+                "commit_id": 3,
             },
         )
 
@@ -1079,6 +1085,7 @@ class TestDiffDatabaseMappingAdd(unittest.TestCase):
                 "metadata_value": "My metadata.",
                 "metadata_id": 1,
                 "id": 1,
+                "commit_id": 3,
             },
         )
 
@@ -1108,12 +1115,15 @@ class TestDiffDatabaseMappingAdd(unittest.TestCase):
                 "metadata_value": "My metadata.",
                 "metadata_id": 1,
                 "id": 1,
+                "commit_id": 3,
             },
         )
 
     def test_add_parameter_value_metadata_doesnt_raise_with_empty_cache(self):
         items, errors = self._db_map.add_parameter_value_metadata(
-            {"parameter_value_id": 1, "metadata_id": 1, "alternative_id": 1}, cache={}, strict=False
+            {"parameter_value_id": 1, "metadata_id": 1, "alternative_id": 1},
+            cache=DBCache(lambda *args, **kwargs: None),
+            strict=False,
         )
         self.assertEqual(items, set())
         self.assertEqual(len(errors), 1)
@@ -1149,6 +1159,7 @@ class TestDiffDatabaseMappingAdd(unittest.TestCase):
                 "metadata_value": "parameter metadata",
                 "metadata_id": 1,
                 "id": 1,
+                "commit_id": 3,
             },
         )
 
@@ -1182,6 +1193,7 @@ class TestDiffDatabaseMappingAdd(unittest.TestCase):
                 "metadata_value": "My metadata.",
                 "metadata_id": 1,
                 "id": 1,
+                "commit_id": 3,
             },
         )
 
