@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Database API.
 # Spine Database API is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
@@ -17,6 +17,8 @@ Unit tests for import_functions.py.
 """
 
 import unittest
+
+from spinedb_api.spine_db_server import _unparse_value
 from spinedb_api.diff_db_mapping import DiffDatabaseMapping
 from spinedb_api.db_mapping import DatabaseMapping
 from spinedb_api.import_functions import (
@@ -258,6 +260,20 @@ class TestImportObjectClassParameter(unittest.TestCase):
         self.assertFalse(errors)
         db_map.connection.close()
 
+    def test_import_object_class_parameter_with_null_default_value_and_db_server_unparsing(self):
+        db_map = DatabaseMapping("sqlite://", create=True)
+        import_object_classes(db_map, ["object_class"])
+        _, errors = import_object_parameters(
+            db_map, [["object_class", "parameter", [None, None]]], unparse_value=_unparse_value
+        )
+        self.assertEqual(errors, [])
+        db_map.commit_session("Add test data.")
+        parameters = db_map.query(db_map.object_parameter_definition_sq).all()
+        self.assertEqual(len(parameters), 1)
+        self.assertIsNone(parameters[0].default_value)
+        self.assertIsNone(parameters[0].default_type)
+        db_map.connection.close()
+
 
 class TestImportRelationshipClassParameter(unittest.TestCase):
     def test_import_valid_relationship_class_parameter(self):
@@ -391,6 +407,113 @@ class TestImportRelationship(unittest.TestCase):
         db_map.connection.close()
 
 
+class TestImportParameterDefinition(unittest.TestCase):
+    def setUp(self):
+        self._db_map = DatabaseMapping("sqlite://", create=True)
+
+    def tearDown(self):
+        self._db_map.connection.close()
+
+    def test_import_object_parameter_definition(self):
+        import_object_classes(self._db_map, ["my_object_class"])
+        count, errors = import_object_parameters(self._db_map, (("my_object_class", "my_parameter"),))
+        self.assertEqual(errors, [])
+        self.assertEqual(count, 1)
+        self._db_map.commit_session("Add test data.")
+        parameter_definitions = [
+            row._asdict() for row in self._db_map.query(self._db_map.object_parameter_definition_sq)
+        ]
+        self.assertEqual(
+            parameter_definitions,
+            [
+                {
+                    "default_type": None,
+                    "default_value": None,
+                    "description": None,
+                    "entity_class_id": 1,
+                    "entity_class_name": "my_object_class",
+                    "id": 1,
+                    "object_class_id": 1,
+                    "object_class_name": "my_object_class",
+                    "parameter_name": "my_parameter",
+                    "value_list_id": None,
+                    "value_list_name": None,
+                }
+            ],
+        )
+
+    def test_import_object_parameter_definition_with_value_list(self):
+        import_object_classes(self._db_map, ["my_object_class"])
+        import_parameter_value_lists(self._db_map, (("my_list", 99.0),))
+        count, errors = import_object_parameters(self._db_map, (("my_object_class", "my_parameter", None, "my_list"),))
+        self.assertEqual(errors, [])
+        self.assertEqual(count, 1)
+        self._db_map.commit_session("Add test data.")
+        parameter_definitions = [
+            row._asdict() for row in self._db_map.query(self._db_map.object_parameter_definition_sq)
+        ]
+        self.assertEqual(
+            parameter_definitions,
+            [
+                {
+                    "default_type": None,
+                    "default_value": b"null",
+                    "description": None,
+                    "entity_class_id": 1,
+                    "entity_class_name": "my_object_class",
+                    "id": 1,
+                    "object_class_id": 1,
+                    "object_class_name": "my_object_class",
+                    "parameter_name": "my_parameter",
+                    "value_list_id": 1,
+                    "value_list_name": "my_list",
+                }
+            ],
+        )
+
+    def test_import_object_parameter_definition_with_default_value_from_value_list(self):
+        import_object_classes(self._db_map, ["my_object_class"])
+        import_parameter_value_lists(self._db_map, (("my_list", 99.0),))
+        count, errors = import_object_parameters(self._db_map, (("my_object_class", "my_parameter", 99.0, "my_list"),))
+        self.assertEqual(errors, [])
+        self.assertEqual(count, 1)
+        self._db_map.commit_session("Add test data.")
+        parameter_definitions = [
+            row._asdict() for row in self._db_map.query(self._db_map.object_parameter_definition_sq)
+        ]
+        self.assertEqual(
+            parameter_definitions,
+            [
+                {
+                    "default_type": None,
+                    "default_value": b"99.0",
+                    "description": None,
+                    "entity_class_id": 1,
+                    "entity_class_name": "my_object_class",
+                    "id": 1,
+                    "object_class_id": 1,
+                    "object_class_name": "my_object_class",
+                    "parameter_name": "my_parameter",
+                    "value_list_id": 1,
+                    "value_list_name": "my_list",
+                }
+            ],
+        )
+
+    def test_import_object_parameter_definition_with_default_value_from_value_list_fails_gracefully(self):
+        import_object_classes(self._db_map, ["my_object_class"])
+        import_parameter_value_lists(self._db_map, (("my_list", 99.0),))
+        count, errors = import_object_parameters(self._db_map, (("my_object_class", "my_parameter", 23.0, "my_list"),))
+        self.assertEqual(
+            [error.msg for error in errors],
+            [
+                "Could not import parameter 'my_parameter' with class 'my_object_class': "
+                "Invalid default_value '23.0' - it should be one from the parameter value list: '99.0'."
+            ],
+        )
+        self.assertEqual(count, 0)
+
+
 class TestImportParameterValue(unittest.TestCase):
     @staticmethod
     def populate(db_map):
@@ -810,6 +933,25 @@ class TestImportParameterValueList(unittest.TestCase):
         self.assertEqual(from_database(list_values[0].value, list_values[0].type), 23.0)
         self.assertEqual(list_values[0].index, 0)
 
+    def test_import_twelfth_value(self):
+        n_values = 11
+        initial_list = tuple(("list_1", 1.1 * i) for i in range(1, n_values + 1))
+        count, errors = import_parameter_value_lists(self._db_map, initial_list)
+        self.assertEqual(errors, [])
+        self.assertEqual(count, n_values + 1)
+        count, errors = import_parameter_value_lists(self._db_map, (("list_1", 23.0),))
+        self.assertEqual(errors, [])
+        self.assertEqual(count, 1)
+        value_lists = self._db_map.query(self._db_map.parameter_value_list_sq).all()
+        self.assertEqual(len(value_lists), 1)
+        self.assertEqual(value_lists[0].name, "list_1")
+        list_values = self._db_map.query(self._db_map.list_value_sq).all()
+        self.assertEqual(len(list_values), n_values + 1)
+        expected = {i: 1.1 * (i + 1) for i in range(n_values)}
+        expected[len(expected)] = 23.0
+        for row in list_values:
+            self.assertEqual(from_database(row.value, row.type), expected[row.index])
+
 
 class TestImportAlternative(unittest.TestCase):
     def test_single_alternative(self):
@@ -1167,6 +1309,24 @@ class TestImportToolFeatureMethod(unittest.TestCase):
         self.assertTrue(errors)
         db_map.connection.close()
 
+    def test_tool_feature_method_with_db_server_style_method(self):
+        db_map = DatabaseMapping("sqlite://", create=True)
+        self.populate(db_map)
+        db_map.commit_session("Add test data.")
+        count, errors = import_tool_feature_methods(
+            db_map, [["tool1", "object_class1", "parameter1", [b'"value1"', None]]], unparse_value=_unparse_value
+        )
+        self.assertEqual(errors, [])
+        self.assertEqual(count, 1)
+        tool_feature_methods = db_map.query(db_map.ext_tool_feature_method_sq).all()
+        self.assertEqual(len(tool_feature_methods), 1)
+        self.assertEqual(tool_feature_methods[0].entity_class_name, "object_class1")
+        self.assertEqual(from_database(tool_feature_methods[0].method), "value1")
+        self.assertEqual(tool_feature_methods[0].parameter_definition_name, "parameter1")
+        self.assertEqual(tool_feature_methods[0].parameter_value_list_name, "value_list")
+        self.assertEqual(tool_feature_methods[0].tool_name, "tool1")
+        db_map.connection.close()
+
 
 class TestImportMetadata(unittest.TestCase):
     def test_import_metadata(self):
@@ -1314,6 +1474,7 @@ class TestImportParameterValueMetadata(unittest.TestCase):
                 "metadata_value": "John",
                 "parameter_name": "param",
                 "parameter_value_id": 1,
+                "commit_id": None,
             },
         )
         self.assertEqual(
@@ -1327,6 +1488,7 @@ class TestImportParameterValueMetadata(unittest.TestCase):
                 "metadata_value": "17",
                 "parameter_name": "param",
                 "parameter_value_id": 1,
+                "commit_id": None,
             },
         )
 
@@ -1355,6 +1517,7 @@ class TestImportParameterValueMetadata(unittest.TestCase):
                 "metadata_value": "John",
                 "parameter_name": "param",
                 "parameter_value_id": 1,
+                "commit_id": None,
             },
         )
         self.assertEqual(
@@ -1368,6 +1531,7 @@ class TestImportParameterValueMetadata(unittest.TestCase):
                 "metadata_value": "17",
                 "parameter_name": "param",
                 "parameter_value_id": 1,
+                "commit_id": None,
             },
         )
 
